<?php
namespace Teknisa\GQL\Modules\Cadastros\Controllers;

use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Teknisa\GER\Services\CryptService;

class Gql01200QualificacaoMarcasProdutoController 
{
    
    public function __construct($service, CryptService $cryptService)
    {
        $this->service      = $service;
        $this->cryptService = $cryptService;
    }
    
    public function deleteQualificacaoMarcaProduto(DTO\Request $request, DTO\Response $response)
    {
		try {
            $row = $request->getRow();
            
			$this->service->deleteQualificacaoMarcaProduto($row);
			
			$response->addMessage(new Message("Excluído com sucesso."));
		} catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
		}
	}
	
	public function insertQualificacaoMarcaProduto(DTO\Request $request, DTO\Response $response)
	{
		try {
            $row = $request->getRow();
            
            $marca = $this->service->getMarca($row);
            
            if ($marca) {
            	throw new \Exception("qualificacaoCadastrada");	
            }
            
			$this->service->insertQualificacaoMarcaProduto($row);
			
			if ($row['MSG']=="S") {
				$this->service->updateQualificacao($row);	
			}
			
			$this->service->insertMarcaProduto($row);
			
			$this->service->procInsertQualificacaoMarcaProduto($row);
			
			$response->addMessage(new Message("Salvo com sucesso."));
		} catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
		}
	}
	
	public function updateQualificacaoMarcaProduto(DTO\Request $request, DTO\Response $response)
	{
		try {
            $row = $request->getRow();
            
            $this->service->updateQualificacaoMarcaProduto($row);
			
			if ($row['MSG']=="S") {
				$this->service->insertQualificacaoMarcaProduto($row);
				$this->service->updateQualificacao($row);
				$this->service->insertMarcaProduto($row);
				$this->service->procInsertQualificacaoMarcaProduto($row);
			}
			
			$response->addMessage(new Message("Salvo com sucesso."));
		} catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
		}
	}
	
	public function passwordValidation(DTO\Request $request, DTO\Response $response)
	{
		try {
            $row = $request->getRow();
            
            $operador = $this->service->selectOperador($row);
            $passwordEncrypted = $this->cryptService->getDelphiCrypt($row['PASSWORD']);
            
	        if ($passwordEncrypted != $operador[0]['CDSENHOPER']) {
	            throw new \Exception("Senha inválida", 1);
	        }
		} catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
		}
	}
}