<?php
namespace Teknisa\GQL\Modules\Cadastros\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\Controller;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DTO\Response\Notification;
use Teknisa\GQL\Modules\Cadastros\Services\gql00703QualificacaoDeFornecedoresService;
use Teknisa\GER\Services\CryptService;

class gql00703QualificacaoDeFornecedoresController extends \Zeedhi\Framework\Controller\Report
{
    const TIPO = array(
        "Fornecedor" => "F",
        "Cliente"    => "C",
        "Prospect"   => "P",
        "Fabricante" => "B",
    );

    protected $connection;
    protected $qr2Strategy;
    protected $cryptService;
    protected $stNomeRel;
    protected $stExtensao;
    
    public function __construct (reportService $reportService, gql00703QualificacaoDeFornecedoresService $service, $connection, $qr2Strategy, $cryptService) {
        parent::__construct($reportService);
        $this->connection    = $connection;
        $this->service       = $service;
        $this->qr2Strategy   = $qr2Strategy;
        $this->cryptService  = $cryptService;
    }
    
    public function getValidade(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();
        $rangeData = [];
        
        if($row['TIPOREPORT']){
            
            switch ($row['TIPOREPORT']) {
                case 'F':
                    $rangeData = $this->service->findDataFornecedor();
                    break;
                    
                case 'C':
                    $rangeData = $this->service->findDataCliente();
                    break;
                    
                case 'P':
                    $rangeData = $this->service->findDataProspect();
                    break;
                    
                case 'B':
                    $rangeData = $this->service->findDataFabricante();
                    break;
            }
            
            $rangeData[0]['DTMENOR'] = substr($rangeData[0]['DTMENOR'], 0, 10);
            $rangeData[0]['DTMAIOR'] = substr($rangeData[0]['DTMAIOR'], 0, 10);
            
            $response->addDataSet(new DataSet('data', $rangeData));
        }
    }
    
    public function getJustificativa(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();
        
        switch ($row['IDTIPO']) {
            case 'F':
                $historico = $this->service->getHistoricoQualificacaoFornecedor($row);
                $justificativa['TXHISQUALFOR'] = !empty($historico) ? $historico[0]['TXHISQUALFOR'] : "";
                break;
                
            case 'C':
                $row['CDCLIENTE'] = $row['CDFORNECED']; 
                $historico = $this->service->getHistoricoQualificacaoCliente($row);
                $justificativa['TXHISQUALFOR'] = !empty($historico) ? $historico[0]['TXHISQUALCLI'] : "";
                break;
                
            case 'P':
                $row['CDPROSPECT'] = $row['CDFORNECED'];
                $historico = $this->service->getHistoricoQualificacaoProspect($row);
                $justificativa['TXHISQUALFOR'] = !empty($historico) ? $historico[0]['TXHISQUALPRO'] : "";
                break;
                
            case 'B':
                $row['CDFABRIC'] = $row['CDFORNECED'];
                $historico = $this->service->getHistoricoQualificacaoFabricante($row);
                $justificativa['TXHISQUALFOR'] = !empty($historico) ? $historico[0]['TXHISQUALFAB'] : "";
                break;
        }
        
        $response->addDataSet(new DataSet('res', $justificativa));
    }
    
    public function updateQualificacao(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();
        
        switch ($row['IDTIPO']) {
            case self::TIPO['Fornecedor']:
                $exist =  $this->service->getHistoricoQualificacaoFornecedor($row);
                    if(empty($exist)){
                        $this->service->insertHistoricoQualificacaoFornecedor($row);
                    }
                    else{
                      $this->service->updateHistoricoQualificacaoFornecedor($row);
                     }
                $this->service->insertJustQualificacaoFornecedor($row);
                $this->service->updateQualificacaoFornecedor($row);
                break;
                
            case self::TIPO['Cliente']:
                $row['TXHISQUALCLI'] = $row['TXHISQUALFOR'];
                $row['CDCLIENTE'] = $row['CDFORNECED'];
                 $exist =  $this->service->getHistoricoQualificacaoCliente($row);
                    if(empty($exist)){
                        $this->service->insertHistoricoQualificacaoCliente($row);
                    }
                    else{
                     $this->service->updateHistoricoQualificacaoCliente($row);
                     }
              
                $this->service->insertJustQualificacaoCliente($row);
                $this->service->updateQualificacaoCliente($row);
                break;
                
            case self::TIPO['Prospect']:
                $row['TXHISQUALPRO'] = $row['TXHISQUALFOR'];
                $row['CDPROSPECT'] = $row['CDFORNECED'];
                $exist =  $this->service->getHistoricoQualificacaoProspect($row);
                 if(empty($exist)){
                        $this->service->insertHistoricoQualificacaoProspect($row);
                    }
                    else{
                         $this->service->updateHistoricoQualificacaoProspect($row);
                     }
             
                $this->service->insertJustQualificacaoProspect($row);
                $this->service->updateQualificacaoProspect($row);
                break;
                
            case self::TIPO['Fabricante']:
                $row['TXHISQUALFAB'] = $row['TXHISQUALFOR'];
                $row['CDFABRIC'] = $row['CDFORNECED'];
                $exist =  $this->service->getHistoricoQualificacaoFabricante($row);
                
                 if(empty($exist)){
                        $this->service->insertHistoricoQualificacaoFabricante($row);
                    }
                    else{
                        $this->service->updateHistoricoQualificacaoFabricante($row);
                     }
              
                $this->service->insertJustQualificacaoFabricante($row);
                $this->service->updateQualificacaoFabricante($row);
                break;
        }
    }
    
    public function alteracaoAutomatica(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();
        
        switch ($row['TIPOREPORT']) {
            case self::TIPO['Fornecedor']:
                $alteracao = self::alteracaoAutomaticaFornecedor($row);
                break;
            
            case self::TIPO['Cliente']:
                $alteracao = self::alteracaoAutomaticaCliente($row);
                break;
                
            case self::TIPO['Prospect']:
                $alteracao = self::alteracaoAutomaticaProspect($row);
                break;
                
            case self::TIPO['Fabricante']:
                $alteracao =  self::alteracaoAutomaticaFabricante($row);
                break;
        }
        
        $response->addDataSet(new DataSet('res', [$alteracao]));
     
    }
    
    public function alteracaoAutomaticaFornecedor($row)
    {
        
        $this->service->insertLogQualificacaoFornecedor($row);
        $fornecedores = $this->service->getHistoricoAlteracaoFornecedor($row);
        
        if (!empty($fornecedores)) {
            
            foreach ($fornecedores as $fornecedor) {
                
                $qualificacao = $this->service->getHistoricoQualificacaoFornecedor($fornecedor);
                $fornecedor['TXHISQUALFOR'] = $row['TXHISQUALFOR'];
                if (empty($qualificacao)) {
                    $this->service->insertHistoricoQualificacaoFornecedor($fornecedor);
                } else {
                    $this->service->updateHistoricoQualificacaoFornecedor($fornecedor);
                }
                
                $this->service->updateDataQualificacaoFornecedor($row);
            }
        } else {
            $message = 'Nenhuma qualificação foi alterada!';
            return $message;
            
        }
    }
    
    public function alteracaoAutomaticaCliente($row)
    {
        $this->service->insertLogQualificacaoCliente($row);
        $clientes = $this->service->getHistoricoAlteracaoCliente($row);
        if (!empty($clientes)) {
            
            foreach ($clientes as $cliente) {
                
                $qualificacao = $this->service->getHistoricoQualificacaoCliente($cliente);
                $cliente['TXHISQUALCLI'] = $row['TXHISQUALFOR'];
                if (empty($qualificacao)) {
                    $this->service->insertHistoricoQualificacaoCliente($cliente);
                } else {
                    $this->service->updateHistoricoQualificacaoCliente($cliente);
                }
                
                $this->service->updateDataQualificacaoCliente($row);
            }
        } else {
            $message = 'Nenhuma qualificação foi alterada!';
            return $message;
            
        }
    }
    
    public function alteracaoAutomaticaProspect($row)
    {
        $this->service->insertLogQualificacaoProspect($row);
        $Prospects = $this->service->getHistoricoAlteracaoProspect($row);
        
        if (!empty($Prospects)) {
            
            foreach ($Prospects as $Prospect) {
                
                $qualificacao = $this->service->getHistoricoQualificacaoProspect($Prospect);
                $Prospect['TXHISQUALPRO'] = $row['TXHISQUALFOR'];
                if (empty($qualificacao)) {
                    $this->service->insertHistoricoQualificacaoProspect($Prospect);
                } else {
                    $this->service->updateHistoricoQualificacaoProspect($Prospect);
                }
                
                $this->service->updateDataQualificacaoProspect($row);
            }
        }
        else {
            $message = 'Nenhuma qualificação foi alterada!';
            return $message;
        }
    }
    
    public function alteracaoAutomaticaFabricante($row)
    {
        $this->service->insertLogQualificacaoFabricante($row);
        $fabricantes = $this->service->getHistoricoAlteracaoFabricante($row);
        if (!empty($fabricantes)) {
            
            foreach ($fabricantes as $fabricante) {
                
                $qualificacao = $this->service->getHistoricoQualificacaoFabricante($fabricante);
                $fabricante['TXHISQUALFAB'] = $row['TXHISQUALFOR'];
                if (empty($qualificacao)) {
                    $this->service->insertHistoricoQualificacaoFabricante($fabricante);
                } else {
                    $this->service->updateHistoricoQualificacaoFabricante($fabricante);
                }
                
                $this->service->updateDataQualificacaoFabricante($row);
            }
            
        } else {
            $message = 'Nenhuma qualificação foi alterada!';
            return $message;
            
        }
    }
    
    public function passwordValidation(DTO\Request $request, DTO\Response $response)
	{
		try {
            $row = $request->getRow();
            
            $operador = $this->service->selectOperador($row);
            $passwordEncrypted = $this->cryptService->getDelphiCrypt($row['PASSWORD']);
            
	        if ($passwordEncrypted != $operador[0]['CDSENHOPER']) {
	            throw new \Exception("Senha inválida", 1);
	        }
		} catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
		}
	}
    
    protected $reportMapping = array ();
    
    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();
        $this->qr2Strategy->setQr2Type($row['QR2TYPE']);
        $this->stNomeRel = $this->getReportName($row);
        $row['CDFORNECED1'] = $row['CDFORNECED'];
        $row['CDFORNECED2'] = $row['CDFORNECED'];
        $row['CDFORNECED3'] = $row['CDFORNECED'];
        $this->reportMapping[$this->stNomeRel] =
        array (
            'strategy' => $row["STRATEGY"],
            'parameterMapping' => 
            array (
                'DTINI'         => 'P_DTINI',
                'DTFIN'         => 'P_DTFIN',
                'DTVALIDADE'    => 'P_DTVALIDADE',
                'CDFORNECED'    => 'L_CDFORNECED',
                'TIPOREPORT'    => 'P_IDTIPO',
                'IDCONSIALVARA' => 'P_CONSAVARA',
                'CDFORNECED1'   => 'L_CDFORNECED1',
                'CDFORNECED2'   => 'L_CDFORNECED2',
                'CDFORNECED3'   => 'L_CDFORNECED3',
            ),
        );
    }
}